@@ -1,35 +1,15 @@
 /*
- * Copyright 1989 - 1994, Julianne Frances Haugh
- * All rights reserved.
+ * SPDX-FileCopyrightText: 1989 - 1994, Julianne Frances Haugh
+ * SPDX-FileCopyrightText: 1996 - 1997, Marek Michałkiewicz
+ * SPDX-FileCopyrightText: 2005       , Tomasz Kłoczko
+ * SPDX-FileCopyrightText: 2008       , Nicolas François
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of Julianne F. Haugh nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY JULIE HAUGH AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL JULIE HAUGH OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-3-Clause
  */
 
 #include <config.h>
 
-#ident "$Id: port.c 1672 2008-01-06 13:42:47Z nekral-guest $"
+#ident "$Id$"
 
 #include <stdio.h>
 #include <ctype.h>
@@ -54,15 +34,19 @@ static int portcmp (const char *pattern, const char *port)
 {
 	const char *orig = port;
 
-	while (*pattern && *pattern == *port)
-		pattern++, port++;
+	while (('\0' != *pattern) && (*pattern == *port)) {
+		pattern++;
+		port++;
+	}
 
-	if (*pattern == 0 && *port == 0)
+	if (('\0' == *pattern) && ('\0' == *port)) {
 		return 0;
-	if (orig[0] == 'S' && orig[1] == 'U' && orig[2] == '\0')
+	}
+	if (('S' == orig[0]) && ('U' == orig[1]) && ('\0' == orig[2])) {
 		return 1;
+	}
 
-	return *pattern == '*' ? 0 : 1;
+	return (*pattern == '*') ? 0 : 1;
 }
 
 /*
@@ -74,10 +58,11 @@ static int portcmp (const char *pattern, const char *port)
 
 static void setportent (void)
 {
-	if (ports)
+	if (NULL != ports) {
 		rewind (ports);
-	else
+	} else {
 		ports = fopen (PORTS, "r");
+	}
 }
 
 /*
@@ -90,10 +75,11 @@ static void setportent (void)
 
 static void endportent (void)
 {
-	if (ports)
-		fclose (ports);
+	if (NULL != ports) {
+		(void) fclose (ports);
+	}
 
-	ports = (FILE *) 0;
+	ports = NULL;
 }
 
 /*
@@ -122,10 +108,11 @@ static struct port *getportent (void)
 	 * since we want to search from the beginning each time.
 	 */
 
-	if (!ports)
+	if (NULL == ports) {
 		setportent ();
+	}
 
-	if (!ports) {
+	if (NULL == ports) {
 		errno = saveerr;
 		return 0;
 	}
@@ -143,16 +130,17 @@ static struct port *getportent (void)
       again:
 
 	/*
-	 * Get the next line and remove the last character, which
-	 * is a '\n'.  Lines which begin with '#' are all ignored.
+	 * Get the next line and remove optional trailing '\n'.
+	 * Lines which begin with '#' are all ignored.
 	 */
 
 	if (fgets (buf, sizeof buf, ports) == 0) {
 		errno = saveerr;
 		return 0;
 	}
-	if (buf[0] == '#')
+	if ('#' == buf[0]) {
 		goto again;
+	}
 
 	/*
 	 * Get the name of the TTY device.  It is the first colon
@@ -161,51 +149,61 @@ static struct port *getportent (void)
 	 * TTY devices.
 	 */
 
-	buf[strlen (buf) - 1] = 0;
+	buf[strcspn (buf, "\n")] = 0;
 
 	port.pt_names = ttys;
 	for (cp = buf, j = 0; j < PORT_TTY; j++) {
 		port.pt_names[j] = cp;
-		while (*cp && *cp != ':' && *cp != ',')
+		while (('\0' != *cp) && (':' != *cp) && (',' != *cp)) {
 			cp++;
+		}
 
-		if (!*cp)
+		if ('\0' == *cp) {
 			goto again;	/* line format error */
+		}
 
-		if (*cp == ':')	/* end of tty name list */
+		if (':' == *cp) {	/* end of tty name list */
 			break;
+		}
 
-		if (*cp == ',')	/* end of current tty name */
+		if (',' == *cp) {	/* end of current tty name */
 			*cp++ = '\0';
+		}
 	}
-	*cp++ = 0;
-	port.pt_names[j + 1] = (char *) 0;
+	*cp = '\0';
+	cp++;
+	port.pt_names[j + 1] = NULL;
 
 	/*
 	 * Get the list of user names.  It is the second colon
 	 * separated field, and is a comma separated list of user
 	 * names.  The entry '*' is used to specify all usernames.
-	 * The last entry in the list is a (char *) 0 pointer.
+	 * The last entry in the list is a NULL pointer.
 	 */
 
-	if (*cp != ':') {
+	if (':' != *cp) {
 		port.pt_users = users;
 		port.pt_users[0] = cp;
 
-		for (j = 1; *cp != ':'; cp++) {
-			if (*cp == ',' && j < PORT_IDS) {
-				*cp++ = 0;
-				port.pt_users[j++] = cp;
+		for (j = 1; ':' != *cp; cp++) {
+			if ((',' == *cp) && (j < PORT_IDS)) {
+				*cp = '\0';
+				cp++;
+				port.pt_users[j] = cp;
+				j++;
 			}
 		}
 		port.pt_users[j] = 0;
-	} else
+	} else {
 		port.pt_users = 0;
+	}
 
-	if (*cp != ':')
+	if (':' != *cp) {
 		goto again;
+	}
 
-	*cp++ = 0;
+	*cp = '\0';
+	cp++;
 
 	/*
 	 * Get the list of valid times.  The times field is the third
@@ -220,7 +218,7 @@ static struct port *getportent (void)
 	 * the starting time.  Days are presumed to wrap at 0000.
 	 */
 
-	if (*cp == '\0') {
+	if ('\0' == *cp) {
 		port.pt_times = 0;
 		return &port;
 	}
@@ -231,7 +229,7 @@ static struct port *getportent (void)
 	 * Get the next comma separated entry
 	 */
 
-	for (j = 0; *cp && j < PORT_TIMES; j++) {
+	for (j = 0; ('\0' != *cp) && (j < PORT_TIMES); j++) {
 
 		/*
 		 * Start off with no days of the week
@@ -245,7 +243,7 @@ static struct port *getportent (void)
 		 * week or the other two values.
 		 */
 
-		for (i = 0; cp[i] && cp[i + 1] && isalpha (cp[i]); i += 2) {
+		for (i = 0; isalpha(cp[i]) && ('\0' != cp[i + 1]); i += 2) {
 			switch ((cp[i] << 8) | (cp[i + 1])) {
 			case ('S' << 8) | 'u':
 				port.pt_times[j].t_days |= 01;
@@ -284,8 +282,9 @@ static struct port *getportent (void)
 		 * The default is 'Al' if no days were seen.
 		 */
 
-		if (i == 0)
+		if (0 == i) {
 			port.pt_times[j].t_days = 0177;
+		}
 
 		/*
 		 * The start and end times are separated from each
@@ -293,19 +292,25 @@ static struct port *getportent (void)
 		 * representing the times of day.
 		 */
 
-		for (dtime = 0; cp[i] && isdigit (cp[i]); i++)
+		for (dtime = 0; isdigit (cp[i]); i++) {
 			dtime = dtime * 10 + cp[i] - '0';
+		}
 
-		if (cp[i] != '-' || dtime > 2400 || dtime % 100 > 59)
+		if (('-' != cp[i]) || (dtime > 2400) || ((dtime % 100) > 59)) {
 			goto again;
+		}
 		port.pt_times[j].t_start = dtime;
 		cp = cp + i + 1;
 
-		for (dtime = i = 0; cp[i] && isdigit (cp[i]); i++)
+		for (dtime = 0, i = 0; isdigit (cp[i]); i++) {
 			dtime = dtime * 10 + cp[i] - '0';
+		}
 
-		if ((cp[i] != ',' && cp[i]) || dtime > 2400 || dtime % 100 > 59)
+		if (   ((',' != cp[i]) && ('\0' != cp[i]))
+		    || (dtime > 2400)
+		    || ((dtime % 100) > 59)) {
 			goto again;
+		}
 
 		port.pt_times[j].t_end = dtime;
 		cp = cp + i + 1;
@@ -337,24 +342,32 @@ static struct port *getttyuser (const char *tty, const char *user)
 
 	setportent ();
 
-	while ((port = getportent ())) {
-		if (port->pt_names == 0 || port->pt_users == 0)
+	while ((port = getportent ()) != NULL) {
+		if (   (0 == port->pt_names)
+		    || (0 == port->pt_users)) {
 			continue;
+		}
 
-		for (i = 0; port->pt_names[i]; i++)
-			if (portcmp (port->pt_names[i], tty) == 0)
+		for (i = 0; NULL != port->pt_names[i]; i++) {
+			if (portcmp (port->pt_names[i], tty) == 0) {
 				break;
+			}
+		}
 
-		if (port->pt_names[i] == 0)
+		if (port->pt_names[i] == 0) {
 			continue;
+		}
 
-		for (j = 0; port->pt_users[j]; j++)
-			if (strcmp (user, port->pt_users[j]) == 0 ||
-			    strcmp (port->pt_users[j], "*") == 0)
+		for (j = 0; NULL != port->pt_users[j]; j++) {
+			if (   (strcmp (user, port->pt_users[j]) == 0)
+			    || (strcmp (port->pt_users[j], "*") == 0)) {
 				break;
+			}
+		}
 
-		if (port->pt_users[j] != 0)
+		if (port->pt_users[j] != 0) {
 			break;
+		}
 	}
 	endportent ();
 	return port;
@@ -367,7 +380,7 @@ static struct port *getttyuser (const char *tty, const char *user)
  *	the user name and TTY given.
  */
 
-int isttytime (const char *id, const char *port, time_t when)
+bool isttytime (const char *id, const char *port, time_t when)
 {
 	int i;
 	int dtime;
@@ -376,24 +389,27 @@ int isttytime (const char *id, const char *port, time_t when)
 
 	/*
 	 * Try to find a matching entry for this user.  Default to
-	 * letting the user in - there are pleny of ways to have an
+	 * letting the user in - there are plenty of ways to have an
 	 * entry to match all users.
 	 */
 
-	if (!(pp = getttyuser (port, id)))
-		return 1;
+	pp = getttyuser (port, id);
+	if (NULL == pp) {
+		return true;
+	}
 
 	/*
 	 * The entry is there, but has no time entries - don't
 	 * ever let them login.
 	 */
 
-	if (pp->pt_times == 0)
-		return 0;
+	if (0 == pp->pt_times) {
+		return false;
+	}
 
 	/*
 	 * The current time is converted to HHMM format for
-	 * comparision against the time values in the TTY entry.
+	 * comparison against the time values in the TTY entry.
 	 */
 
 	tm = localtime (&when);
@@ -402,22 +418,25 @@ int isttytime (const char *id, const char *port, time_t when)
 	/*
 	 * Each time entry is compared against the current
 	 * time.  For entries with the start after the end time,
-	 * the comparision is made so that the time is between
+	 * the comparison is made so that the time is between
 	 * midnight and either the start or end time.
 	 */
 
 	for (i = 0; pp->pt_times[i].t_start != -1; i++) {
-		if (!(pp->pt_times[i].t_days & PORT_DAY (tm->tm_wday)))
+		if (!(pp->pt_times[i].t_days & PORT_DAY (tm->tm_wday))) {
 			continue;
+		}
 
 		if (pp->pt_times[i].t_start <= pp->pt_times[i].t_end) {
-			if (dtime >= pp->pt_times[i].t_start &&
-			    dtime <= pp->pt_times[i].t_end)
-				return 1;
+			if (   (dtime >= pp->pt_times[i].t_start)
+			    && (dtime <= pp->pt_times[i].t_end)) {
+				return true;
+			}
 		} else {
-			if (dtime >= pp->pt_times[i].t_start ||
-			    dtime <= pp->pt_times[i].t_end)
-				return 1;
+			if (   (dtime >= pp->pt_times[i].t_start)
+			    || (dtime <= pp->pt_times[i].t_end)) {
+				return true;
+			}
 		}
 	}
 
@@ -426,5 +445,6 @@ int isttytime (const char *id, const char *port, time_t when)
 	 * be let in right now.
 	 */
 
-	return 0;
+	return false;
 }
+
