@@ -29,7 +29,6 @@
  */
 const char *Prog;
 
-static char name[BUFSIZ];
 static char pass[BUFSIZ];
 
 static struct passwd pwent;
@@ -37,27 +36,18 @@ static struct passwd pwent;
 extern char **newenvp;
 extern size_t newenvc;
 
-extern char **environ;
-
 #ifndef	ALARM
 #define	ALARM	60
 #endif
 
 /* local function prototypes */
-static RETSIGTYPE catch_signals (int);
+static void catch_signals (int);
 
-static RETSIGTYPE catch_signals (unused int sig)
+static void catch_signals (unused int sig)
 {
 	_exit (1);
 }
 
-/*
- * syslogd is usually not running at the time when sulogin is typically
- * called, cluttering the screen with unnecessary messages. Suggested by
- * Ivan Nejgebauer <ian@unsux.ns.ac.yu>.  --marekm
- */
-#undef USE_SYSLOG
-
  /*ARGSUSED*/ int main (int argc, char **argv)
 {
 #ifndef USE_PAM
@@ -67,21 +57,11 @@ static RETSIGTYPE catch_signals (unused int sig)
 	TERMIO termio;
 	int err = 0;
 
-#ifdef	USE_TERMIO
-	ioctl (0, TCGETA, &termio);
-	termio.c_iflag |= (ICRNL | IXON);
-	termio.c_oflag |= (OPOST | ONLCR);
-	termio.c_cflag |= (CREAD);
-	termio.c_lflag |= (ISIG | ICANON | ECHO | ECHOE | ECHOK);
-	ioctl (0, TCSETAF, &termio);
-#endif
-#ifdef	USE_TERMIOS
 	tcgetattr (0, &termio);
 	termio.c_iflag |= (ICRNL | IXON);
 	termio.c_oflag |= (CREAD);
 	termio.c_lflag |= (ECHO | ECHOE | ECHOK | ICANON | ISIG);
 	tcsetattr (0, TCSANOW, &termio);
-#endif
 
 	Prog = Basename (argv[0]);
 	log_set_progname(Prog);
@@ -90,9 +70,6 @@ static RETSIGTYPE catch_signals (unused int sig)
 	(void) bindtextdomain (PACKAGE, LOCALEDIR);
 	(void) textdomain (PACKAGE);
 
-#ifdef	USE_SYSLOG
-	OPENLOG ("sulogin");
-#endif
 	initenv ();
 	if (argc > 1) {
 		close (0);
@@ -103,34 +80,19 @@ static RETSIGTYPE catch_signals (unused int sig)
 			dup (0);
 			dup (0);
 		} else {
-#ifdef	USE_SYSLOG
-			SYSLOG (LOG_WARN, "cannot open %s\n", argv[1]);
-			closelog ();
-#endif
 			exit (1);
 		}
 	}
 	if (access (PASSWD_FILE, F_OK) == -1) {	/* must be a password file! */
 		(void) puts (_("No password file"));
-#ifdef	USE_SYSLOG
-		SYSLOG (LOG_WARN, "No password file\n");
-		closelog ();
-#endif
 		exit (1);
 	}
 #if !defined(DEBUG) && defined(SULOGIN_ONLY_INIT)
 	if (getppid () != 1) {	/* parent must be INIT */
-#ifdef	USE_SYSLOG
-		SYSLOG (LOG_WARN, "Pid == %d, not 1\n", getppid ());
-		closelog ();
-#endif
 		exit (1);
 	}
 #endif
 	if ((isatty (0) == 0) || (isatty (1) == 0) || (isatty (2) == 0)) {
-#ifdef	USE_SYSLOG
-		closelog ();
-#endif
 		exit (1);	/* must be a terminal */
 	}
 	/* If we were init, we need to start a new session */
@@ -156,23 +118,17 @@ static RETSIGTYPE catch_signals (unused int sig)
 	}
 #endif				/* !USE_PAM */
 
-	(void) strcpy (name, "root");	/* KLUDGE!!! */
-
 	(void) signal (SIGALRM, catch_signals);	/* exit if the timer expires */
 	(void) alarm (ALARM);		/* only wait so long ... */
 
 	while (true) {		/* repeatedly get login/password pairs */
 		char *cp;
-		pw_entry (name, &pwent);	/* get entry from password file */
-		if (pwent.pw_name == (char *) 0) {
+		pw_entry ("root", &pwent);	/* get entry from password file */
+		if (pwent.pw_name == NULL) {
 			/*
 			 * Fail secure
 			 */
 			(void) puts (_("No password entry for 'root'"));
-#ifdef	USE_SYSLOG
-			SYSLOG (LOG_WARN, "No password entry for 'root'\n");
-			closelog ();
-#endif
 			exit (1);
 		}
 
@@ -182,7 +138,7 @@ static RETSIGTYPE catch_signals (unused int sig)
 		 */
 
 		/* get a password for root */
-		cp = getpass (_(
+		cp = agetpass (_(
 "\n"
 "Type control-d to proceed with normal startup,\n"
 "(or give root password for system maintenance):"));
@@ -193,44 +149,32 @@ static RETSIGTYPE catch_signals (unused int sig)
 		 * --marekm
 		 */
 		if ((NULL == cp) || ('\0' == *cp)) {
-#ifdef	USE_SYSLOG
-			SYSLOG (LOG_INFO, "Normal startup\n");
-			closelog ();
-#endif
+			erase_pass (cp);
 			(void) puts ("");
 #ifdef	TELINIT
-			execl (PATH_TELINIT, "telinit", RUNLEVEL, (char *) 0);
+			execl (PATH_TELINIT, "telinit", RUNLEVEL, (char *) NULL);
 #endif
 			exit (0);
-		} else {
-			STRFCPY (pass, cp);
-			strzero (cp);
 		}
+		STRFCPY (pass, cp);
+		erase_pass (cp);
+
 		if (valid (pass, &pwent)) {	/* check encrypted passwords ... */
 			break;	/* ... encrypted passwords matched */
 		}
 
-#ifdef	USE_SYSLOG
-		SYSLOG (LOG_WARN, "Incorrect root password\n");
-#endif
 		sleep (2);
 		(void) puts (_("Login incorrect"));
 	}
-	strzero (pass);
+	memzero (pass, sizeof pass);
 	(void) alarm (0);
 	(void) signal (SIGALRM, SIG_DFL);
 	environ = newenvp;	/* make new environment active */
 
 	(void) puts (_("Entering System Maintenance Mode"));
-#ifdef	USE_SYSLOG
-	SYSLOG (LOG_INFO, "System Maintenance Mode\n");
-#endif
 
-#ifdef	USE_SYSLOG
-	closelog ();
-#endif
 	/* exec the shell finally. */
-	err = shell (pwent.pw_shell, (char *) 0, environ);
+	err = shell (pwent.pw_shell, NULL, environ);
 
 	return ((err == ENOENT) ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
 }
