@@ -82,7 +82,7 @@
 /*
  * Global variables
  */
-static const char Prog[] = "useradd";
+const char *Prog;
 
 /*
  * These defaults are used if there is no defaults file.
@@ -214,6 +214,7 @@ static struct group * get_local_group (char * grp_name);
 static void usage (int status);
 static void new_pwent (struct passwd *);
 
+static long scale_age (long);
 static void new_spent (struct spwd *);
 static void grp_update (void);
 
@@ -1007,6 +1008,15 @@ static void new_pwent (struct passwd *pwent)
 	pwent->pw_shell = (char *) user_shell;
 }
 
+static long scale_age (long x)
+{
+	if (x <= 0) {
+		return x;
+	}
+
+	return x * (DAY / SCALE);
+}
+
 /*
  * new_spent - initialize the values in a shadow password file entry
  *
@@ -1018,17 +1028,17 @@ static void new_spent (struct spwd *spent)
 	memzero (spent, sizeof *spent);
 	spent->sp_namp = (char *) user_name;
 	spent->sp_pwdp = (char *) user_pass;
-	spent->sp_lstchg = gettime () / DAY;
+	spent->sp_lstchg = gettime () / SCALE;
 	if (0 == spent->sp_lstchg) {
 		/* Better disable aging than requiring a password change */
 		spent->sp_lstchg = -1;
 	}
 	if (!rflg) {
-		spent->sp_min = getdef_num ("PASS_MIN_DAYS", -1);
-		spent->sp_max = getdef_num ("PASS_MAX_DAYS", -1);
-		spent->sp_warn = getdef_num ("PASS_WARN_AGE", -1);
-		spent->sp_inact = def_inactive;
-		spent->sp_expire = user_expire;
+		spent->sp_min = scale_age (getdef_num ("PASS_MIN_DAYS", -1));
+		spent->sp_max = scale_age (getdef_num ("PASS_MAX_DAYS", -1));
+		spent->sp_warn = scale_age (getdef_num ("PASS_WARN_AGE", -1));
+		spent->sp_inact = scale_age (def_inactive);
+		spent->sp_expire = scale_age (user_expire);
 	} else {
 		spent->sp_min = -1;
 		spent->sp_max = -1;
@@ -2518,6 +2528,10 @@ int main (int argc, char **argv)
 	unsigned long subuid_count = 0;
 	unsigned long subgid_count = 0;
 
+	/*
+	 * Get my name so that I can use it to report errors.
+	 */
+	Prog = Basename (argv[0]);
 	log_set_progname(Prog);
 	log_set_logfd(stderr);
 
@@ -2529,7 +2543,7 @@ int main (int argc, char **argv)
 
 	prefix = process_prefix_flag("-P", argc, argv);
 
-	OPENLOG (Prog);
+	OPENLOG ("useradd");
 #ifdef WITH_AUDIT
 	audit_help_open ();
 #endif
@@ -2581,7 +2595,7 @@ int main (int argc, char **argv)
 			fail_exit (1);
 		}
 
-		retval = pam_start (Prog, pampw?pampw->pw_name:"root", &conv, &pamh);
+		retval = pam_start ("useradd", pampw?pampw->pw_name:"root", &conv, &pamh);
 	}
 
 	if (PAM_SUCCESS == retval) {
